@@ -34,6 +34,8 @@
 #include "androidglobal.h"
 #include "androidpackagecreationwidget.h"
 #include "androidmanager.h"
+#include "androidgdbserverkitinformation.h"
+#include "androidtoolchain.h"
 
 #include <projectexplorer/buildsteplist.h>
 #include <projectexplorer/projectexplorerconstants.h>
@@ -42,6 +44,7 @@
 #include <qt4projectmanager/qt4buildconfiguration.h>
 #include <qt4projectmanager/qt4project.h>
 #include <qt4projectmanager/qt4nodes.h>
+#include <qtsupport/qtkitinformation.h>
 
 #include <coreplugin/icore.h>
 #include <coreplugin/fileutils.h>
@@ -127,6 +130,7 @@ void AndroidPackageCreationStep::ctor()
 {
     setDefaultDisplayName(tr("Packaging for Android"));
     m_openPackageLocation = true;
+    m_bundleQt = false;
     connect(&m_outputParser, SIGNAL(addTask(ProjectExplorer::Task)), this, SIGNAL(addTask(ProjectExplorer::Task)));
 }
 
@@ -143,16 +147,15 @@ bool AndroidPackageCreationStep::init()
     // Copying
     m_androidDir = AndroidManager::dirPath(target());
     Utils::FileName path = m_androidDir;
-    Utils::FileName androidLibPath;
-    if (project->rootQt4ProjectNode()->variableValue(Qt4ProjectManager::ConfigVar).contains(QLatin1String("x86")))
-        androidLibPath = path.appendPath(QLatin1String("libs/x86"));
-    else if (project->rootQt4ProjectNode()
-             ->variableValue(Qt4ProjectManager::ConfigVar).contains(QLatin1String("armeabi-v7a")))
-        androidLibPath = path.appendPath(QLatin1String("libs/armeabi-v7a"));
-    else
-        androidLibPath = path.appendPath(QLatin1String("libs/armeabi"));
+    QString androidTargetArch = project->rootQt4ProjectNode()->singleVariableValue(Qt4ProjectManager::AndroidArchVar);
+    if (androidTargetArch.isEmpty()) {
+        raiseError(tr("Cannot create Android package: No ANDROID_TARGET_ARCH set in make spec."));
+        return false;
+    }
+
+    Utils::FileName androidLibPath = path.appendPath(QLatin1String("libs/") + androidTargetArch);
     m_gdbServerDestination = androidLibPath.appendPath(QLatin1String("gdbserver"));
-    m_gdbServerSource = AndroidConfigurations::instance().gdbServerPath(target()->activeRunConfiguration()->abi().architecture());
+    m_gdbServerSource = AndroidGdbServerKitInformation::gdbServer(target()->kit());
     m_debugBuild = bc->qmakeBuildConfiguration() & QtSupport::BaseQtVersion::DebugBuild;
 
     if (!AndroidManager::createAndroidTemplatesIfNecessary(target()))
@@ -166,6 +169,11 @@ bool AndroidPackageCreationStep::init()
     m_certificatePasswdForRun = m_certificatePasswd;
     m_jarSigner = AndroidConfigurations::instance().jarsignerPath();
     m_zipAligner = AndroidConfigurations::instance().zipalignPath();
+
+    ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(target()->kit());
+    if (tc->type() != QLatin1String(Constants::ANDROID_TOOLCHAIN_TYPE))
+        return false;
+
     initCheckRequiredLibrariesForRun();
     return true;
 }
@@ -192,6 +200,72 @@ static inline QString msgCannotFindExecutable(const QString &appPath)
         "built successfully and is selected in Application tab ('Run option').").arg(appPath);
 }
 
+static void parseSharedLibs(const QByteArray &buffer, QStringList *libs)
+{
+#if defined(_WIN32)
+    QList<QByteArray> lines = buffer.trimmed().split('\r');
+#else
+    QList<QByteArray> lines = buffer.trimmed().split('\n');
+#endif
+    foreach (QByteArray line, lines) {
+        if (line.contains("(NEEDED)") && line.contains("Shared library:") ) {
+            const int pos = line.lastIndexOf('[') + 1;
+            (*libs) << QString::fromLatin1(line.mid(pos, line.length() - pos - 1));
+        }
+    }
+}
+
+void markNeeded(const QString &library,
+                const QVector<AndroidManager::Library> &dependencies,
+                QMap<QString, bool> *neededMap)
+{
+    if (!neededMap->contains(library))
+        return;
+    if (neededMap->value(library))
+        return;
+    neededMap->insert(library, true);
+    for (int i = 0; i < dependencies.size(); ++i) {
+        if (dependencies.at(i).name == library) {
+            foreach (const QString &dependency, dependencies.at(i).dependencies)
+                markNeeded(dependency, dependencies, neededMap);
+            break;
+        }
+    }
+}
+
+QStringList requiredLibraries(QVector<AndroidManager::Library> availableLibraries,
+                      const QStringList &checkedLibs, const QStringList &dependencies)
+{
+    QMap<QString, bool> neededLibraries;
+    QVector<AndroidManager::Library>::const_iterator it, end;
+    it = availableLibraries.constBegin();
+    end = availableLibraries.constEnd();
+
+    for (; it != end; ++it)
+        neededLibraries[(*it).name] = false;
+
+    // Checked items are always needed
+    foreach (const QString &lib, checkedLibs)
+        markNeeded(lib, availableLibraries, &neededLibraries);
+
+    foreach (const QString &lib, dependencies) {
+        if (lib.startsWith(QLatin1String("lib"))
+                && lib.endsWith(QLatin1String(".so")))
+            markNeeded(lib.mid(3, lib.size() - 6), availableLibraries, &neededLibraries);
+    }
+
+    for (int i = availableLibraries.size() - 1; i>= 0; --i)
+        if (!neededLibraries.value(availableLibraries.at(i).name))
+            availableLibraries.remove(i);
+
+    QStringList requiredLibraries;
+    foreach (const AndroidManager::Library &lib, availableLibraries) {
+        if (neededLibraries.value(lib.name))
+            requiredLibraries << lib.name;
+    }
+    return requiredLibraries;
+}
+
 void AndroidPackageCreationStep::checkRequiredLibraries()
 {
     QProcess readelfProc;
@@ -200,47 +274,59 @@ void AndroidPackageCreationStep::checkRequiredLibraries()
         raiseError(msgCannotFindElfInformation(), msgCannotFindExecutable(appPath));
         return;
     }
-    readelfProc.start(AndroidConfigurations::instance().readelfPath(target()->activeRunConfiguration()->abi().architecture()).toString(),
+
+    ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(target()->kit());
+    if (tc->type() != QLatin1String(Constants::ANDROID_TOOLCHAIN_TYPE))
+        return;
+    AndroidToolChain *atc = static_cast<AndroidToolChain *>(tc);
+
+    readelfProc.start(AndroidConfigurations::instance().readelfPath(target()->activeRunConfiguration()->abi().architecture(), atc->ndkToolChainVersion()).toString(),
                       QStringList() << QLatin1String("-d") << QLatin1String("-W") << appPath);
     if (!readelfProc.waitForFinished(-1)) {
-        readelfProc.terminate();
+        readelfProc.kill();
         return;
     }
     QStringList libs;
-    QList<QByteArray> lines = readelfProc.readAll().trimmed().split('\n');
-    foreach (const QByteArray &line, lines) {
-        if (line.contains("(NEEDED)") && line.contains("Shared library:") ) {
-            const int pos = line.lastIndexOf('[') + 1;
-            libs << QString::fromLatin1(line.mid(pos, line.length() - pos - 1));
-        }
-    }
-    QStringList checkedLibs = AndroidManager::qtLibs(target());
-    QStringList requiredLibraries;
-    foreach (const QString &qtLib, AndroidManager::availableQtLibs(target())) {
-        if (libs.contains(QLatin1String("lib") + qtLib + QLatin1String(".so")) || checkedLibs.contains(qtLib))
-            requiredLibraries << qtLib;
-    }
-    AndroidManager::setQtLibs(target(), requiredLibraries);
+    parseSharedLibs(readelfProc.readAll(), &libs);
+    AndroidManager::setQtLibs(target(), requiredLibraries(AndroidManager::availableQtLibsWithDependencies(target()),
+                                                          AndroidManager::qtLibs(target()), libs));
 
-    checkedLibs = AndroidManager::prebundledLibs(target());
-    requiredLibraries.clear();
+    QStringList checkedLibs = AndroidManager::prebundledLibs(target());
+    QStringList prebundledLibraries;
     foreach (const QString &qtLib, AndroidManager::availableQtLibs(target())) {
         if (libs.contains(qtLib) || checkedLibs.contains(qtLib))
-            requiredLibraries << qtLib;
+            prebundledLibraries << qtLib;
     }
-    AndroidManager::setPrebundledLibs(target(), requiredLibraries);
+    AndroidManager::setPrebundledLibs(target(), prebundledLibraries);
     emit updateRequiredLibrariesModels();
 }
 
 void AndroidPackageCreationStep::initCheckRequiredLibrariesForRun()
 {
+    ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(target()->kit());
+    if (tc->type() != QLatin1String(Constants::ANDROID_TOOLCHAIN_TYPE))
+        return;
+    AndroidToolChain *atc = static_cast<AndroidToolChain *>(tc);
+
     m_appPath = Utils::FileName::fromString(AndroidManager::targetApplicationPath(target()));
-    m_readElf = AndroidConfigurations::instance().readelfPath(target()->activeRunConfiguration()->abi().architecture());
+    m_readElf = AndroidConfigurations::instance().readelfPath(target()->activeRunConfiguration()->abi().architecture(),
+                                                              atc->ndkToolChainVersion());
     m_qtLibs = AndroidManager::qtLibs(target());
-    m_availableQtLibs = AndroidManager::availableQtLibs(target());
+    m_availableQtLibs = AndroidManager::availableQtLibsWithDependencies(target());
     m_prebundledLibs = AndroidManager::prebundledLibs(target());
 }
 
+void AndroidPackageCreationStep::getBundleInformation()
+{
+    m_bundleQt = AndroidManager::bundleQt(target());
+    if (m_bundleQt) {
+        m_bundledJars = AndroidManager::loadLocalJars(target()).split(QLatin1Char(':'),
+                                                                      QString::SkipEmptyParts);
+        m_otherBundledFiles = AndroidManager::loadLocalBundledFiles(target()).split(QLatin1Char(':'),
+                                                                                    QString::SkipEmptyParts);
+    }
+}
+
 void AndroidPackageCreationStep::checkRequiredLibrariesForRun()
 {
     QProcess readelfProc;
@@ -250,34 +336,26 @@ void AndroidPackageCreationStep::checkRequiredLibrariesForRun()
     }
     readelfProc.start(m_readElf.toString(), QStringList() << QLatin1String("-d") << QLatin1String("-W") << m_appPath.toUserOutput());
     if (!readelfProc.waitForFinished(-1)) {
-        readelfProc.terminate();
+        readelfProc.kill();
         return;
     }
     QStringList libs;
-    QList<QByteArray> lines = readelfProc.readAll().trimmed().split('\n');
-    foreach (const QByteArray &line, lines) {
-        if (line.contains("(NEEDED)") && line.contains("Shared library:") ) {
-            const int pos = line.lastIndexOf('[') + 1;
-            libs << QString::fromLatin1(line.mid(pos, line.length() - pos - 1));
-        }
-    }
+    parseSharedLibs(readelfProc.readAll(), &libs);
 
-    QStringList requiredLibraries;
-    foreach (const QString &qtLib, m_availableQtLibs) {
-        if (libs.contains(QLatin1String("lib") + qtLib + QLatin1String(".so")) || m_qtLibs.contains(qtLib))
-            requiredLibraries << qtLib;
-    }
+    m_qtLibsWithDependencies = requiredLibraries(m_availableQtLibs, m_qtLibs, libs);
     QMetaObject::invokeMethod(this, "setQtLibs",Qt::BlockingQueuedConnection,
-                              Q_ARG(QStringList, requiredLibraries));
+                              Q_ARG(QStringList, m_qtLibsWithDependencies));
 
-    requiredLibraries.clear();
-    foreach (const QString &qtLib, m_availableQtLibs) {
-        if (libs.contains(qtLib) || m_prebundledLibs.contains(qtLib))
-            requiredLibraries << qtLib;
+    QMetaObject::invokeMethod(this, "getBundleInformation");
+
+    QStringList prebundledLibraries;
+    foreach (const AndroidManager::Library &qtLib, m_availableQtLibs) {
+        if (libs.contains(qtLib.name) || m_prebundledLibs.contains(qtLib.name))
+            prebundledLibraries << qtLib.name;
     }
 
     QMetaObject::invokeMethod(this, "setPrebundledLibs", Qt::BlockingQueuedConnection,
-                              Q_ARG(QStringList, requiredLibraries));
+                              Q_ARG(QStringList, prebundledLibraries));
     emit updateRequiredLibrariesModels();
 }
 
@@ -367,6 +445,160 @@ QVariantMap AndroidPackageCreationStep::toMap() const
     return map;
 }
 
+QStringList AndroidPackageCreationStep::collectRelativeFilePaths(const QString &parentPath)
+{
+    QStringList relativeFilePaths;
+
+    QDirIterator libsIt(parentPath, QDir::NoFilter, QDirIterator::Subdirectories);
+    int pos = parentPath.size();
+    while (libsIt.hasNext()) {
+        libsIt.next();
+        if (!libsIt.fileInfo().isDir())
+            relativeFilePaths.append(libsIt.filePath().mid(pos));
+    }
+
+    return relativeFilePaths;
+}
+
+void AndroidPackageCreationStep::collectFiles(QList<DeployItem> *deployList,
+                                              QList<DeployItem> *pluginsAndImportsList)
+{
+    Q_ASSERT(deployList != 0);
+    QtSupport::BaseQtVersion *version = QtSupport::QtKitInformation::qtVersion(target()->kit());
+    if (!version)
+        return;
+
+    Qt4Project *project = static_cast<Qt4Project *>(target()->project());
+    QString androidTargetArch = project->rootQt4ProjectNode()->singleVariableValue(Qt4ProjectManager::AndroidArchVar);
+
+    QString androidAssetsPath = m_androidDir.toString() + QLatin1String("/assets/");
+    QString androidJarPath = m_androidDir.toString() + QLatin1String("/libs/");
+    QString androidLibPath = m_androidDir.toString() + QLatin1String("/libs/") + androidTargetArch;
+
+    QString qtVersionSourcePath = version->sourcePath().toString();
+
+    foreach (QString qtLib, m_qtLibsWithDependencies) {
+        QString fullPath = qtVersionSourcePath
+            + QLatin1String("/lib/lib")
+            + qtLib
+            + QLatin1String(".so");
+        QString destinationPath = androidLibPath
+                + QLatin1String("/lib")
+                + qtLib
+                + QLatin1String(".so");
+
+        DeployItem deployItem(fullPath, 0, destinationPath, true);
+        deployList->append(deployItem);
+    }
+
+    foreach (QString jar, m_bundledJars) {
+        QString fullPath = qtVersionSourcePath + QLatin1Char('/') + jar;
+        QFileInfo fileInfo(fullPath);
+        if (fileInfo.exists()) {
+            QString destinationPath = androidJarPath
+                    + AndroidManager::libraryPrefix()
+                    + fileInfo.fileName();
+            deployList->append(DeployItem(fullPath, 0, destinationPath, true));
+        }
+    }
+
+    QSet<QString> alreadyListed;
+    foreach (QString bundledFile, m_otherBundledFiles) {
+        if (!bundledFile.endsWith(QLatin1Char('/')))
+            bundledFile.append(QLatin1Char('/'));
+
+        QStringList allFiles = collectRelativeFilePaths(qtVersionSourcePath + QLatin1Char('/') + bundledFile);
+        foreach (QString file, allFiles) {
+            QString fullPath = qtVersionSourcePath + QLatin1Char('/') + bundledFile + QLatin1Char('/') + file;
+            if (alreadyListed.contains(fullPath))
+                continue;
+
+            alreadyListed.insert(fullPath);
+
+            QString garbledFileName;
+            QString destinationPath;
+            bool shouldStrip = false;
+            if (file.endsWith(QLatin1String(".so"))) {
+                garbledFileName = QLatin1String("lib")
+                    + AndroidManager::libraryPrefix()
+                    + QString(bundledFile).replace(QLatin1Char('/'), QLatin1Char('_'))
+                    + QString(file).replace(QLatin1Char('/'), QLatin1Char('_'));
+                destinationPath = androidLibPath + QLatin1Char('/') + garbledFileName;
+                shouldStrip = true;
+            } else {
+                garbledFileName = AndroidManager::libraryPrefix() + bundledFile + file;
+                destinationPath = androidAssetsPath + garbledFileName;
+            }
+
+            deployList->append(DeployItem(fullPath, 0, destinationPath, shouldStrip));
+            pluginsAndImportsList->append(DeployItem(garbledFileName,
+                                                     0,
+                                                     bundledFile + file,
+                                                     shouldStrip));
+        }
+    }
+}
+
+void AndroidPackageCreationStep::removeManagedFilesFromPackage()
+{
+    // Clean up all files managed by Qt Creator
+    {
+        QString androidLibPath = m_androidDir.toString() + QLatin1String("/libs/");
+        QDirIterator dirIt(m_androidDir.toString(), QDirIterator::Subdirectories);
+        while (dirIt.hasNext()) {
+            dirIt.next();
+
+            if (!dirIt.fileInfo().isDir()) {
+                bool isQtLibrary = dirIt.fileInfo().path().startsWith(androidLibPath)
+                        && dirIt.fileName().startsWith(QLatin1String("libQt5"))
+                        && dirIt.fileName().endsWith(QLatin1String(".so"));
+
+                if (dirIt.filePath().contains(AndroidManager::libraryPrefix()) || isQtLibrary)
+                    QFile::remove(dirIt.filePath());
+            }
+        }
+    }
+}
+
+void AndroidPackageCreationStep::copyFilesIntoPackage(const QList<DeployItem> &deployList)
+{
+    foreach (DeployItem item, deployList) {
+        QFileInfo info(item.remoteFileName);
+        if (info.exists())
+            QFile::remove(item.remoteFileName);
+        else
+            QDir().mkpath(info.absolutePath());
+
+        QFile::copy(item.localFileName, item.remoteFileName);
+    }
+}
+
+void AndroidPackageCreationStep::stripFiles(const QList<DeployItem> &deployList)
+{
+
+    QStringList fileList;
+    foreach (DeployItem item, deployList)
+        if (item.needsStrip)
+            fileList.append(item.remoteFileName);
+
+    ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(target()->kit());
+    if (tc->type() != QLatin1String(Constants::ANDROID_TOOLCHAIN_TYPE))
+        return;
+
+    AndroidToolChain *atc = static_cast<AndroidToolChain *>(tc);
+    stripAndroidLibs(fileList,
+                     target()->activeRunConfiguration()->abi().architecture(),
+                     atc->ndkToolChainVersion());
+}
+
+void AndroidPackageCreationStep::updateXmlForFiles(const QStringList &inLibList,
+                                                   const QStringList &inAssetsList)
+{
+    AndroidManager::setBundledInLib(target(), inLibList);
+    AndroidManager::setBundledInAssets(target(), inAssetsList);
+}
+
+
 bool AndroidPackageCreationStep::createPackage()
 {
     checkRequiredLibrariesForRun();
@@ -378,6 +610,8 @@ bool AndroidPackageCreationStep::createPackage()
     QFile::remove(m_gdbServerDestination.toString());
     if (m_debugBuild || !m_certificateAlias.length()) {
         build << QLatin1String("debug");
+        QDir dir;
+        dir.mkpath(m_gdbServerDestination.toFileInfo().absolutePath());
         if (!QFile::copy(m_gdbServerSource.toString(), m_gdbServerDestination.toString())) {
             raiseError(tr("Can't copy gdbserver from '%1' to '%2'").arg(m_gdbServerSource.toUserOutput())
                        .arg(m_gdbServerDestination.toUserOutput()));
@@ -387,6 +621,49 @@ bool AndroidPackageCreationStep::createPackage()
         build << QLatin1String("release");
     }
 
+    QList<DeployItem> deployFiles;
+    QList<DeployItem> importsAndPlugins;
+
+    QtSupport::BaseQtVersion *version = QtSupport::QtKitInformation::qtVersion(target()->kit());
+
+    // Qt 5 supports bundling libraries inside the apk. We guard the code for Qt 5 to be sure we
+    // do not disrupt existing projects.
+    if (version && version->qtVersion() >= QtSupport::QtVersionNumber(5, 0, 0)) {
+        bool bundleQt = AndroidManager::bundleQt(target());
+
+        // Collect the files to bundle in the package
+        if (bundleQt)
+            collectFiles(&deployFiles, &importsAndPlugins);
+
+        // Remove files from package if they are not needed
+        removeManagedFilesFromPackage();
+
+        // Deploy files to package
+        if (bundleQt) {
+            copyFilesIntoPackage(deployFiles);
+            stripFiles(deployFiles);
+
+            QStringList inLibList;
+            QStringList inAssetsList;
+            foreach (DeployItem deployItem, importsAndPlugins) {
+                QString conversionInfo = deployItem.localFileName
+                        + QLatin1Char(':')
+                        + deployItem.remoteFileName;
+
+                if (deployItem.localFileName.endsWith(QLatin1String(".so")))
+                    inLibList.append(conversionInfo);
+                else
+                    inAssetsList.append(conversionInfo);
+            }
+
+            QMetaObject::invokeMethod(this,
+                                      "updateXmlForFiles",
+                                      Qt::BlockingQueuedConnection,
+                                      Q_ARG(QStringList, inLibList),
+                                      Q_ARG(QStringList, inAssetsList));
+        }
+    }
+
     emit addOutput(tr("Creating package file ..."), MessageOutput);
 
     QProcess *const buildProc = new QProcess;
@@ -459,15 +736,15 @@ bool AndroidPackageCreationStep::createPackage()
     return true;
 }
 
-void AndroidPackageCreationStep::stripAndroidLibs(const QStringList & files, Abi::Architecture architecture)
+void AndroidPackageCreationStep::stripAndroidLibs(const QStringList & files, Abi::Architecture architecture, const QString &ndkToolchainVersion)
 {
     QProcess stripProcess;
     foreach (const QString &file, files) {
-        stripProcess.start(AndroidConfigurations::instance().stripPath(architecture).toString(),
+        stripProcess.start(AndroidConfigurations::instance().stripPath(architecture, ndkToolchainVersion).toString(),
                            QStringList()<<QLatin1String("--strip-unneeded") << file);
         stripProcess.waitForStarted();
         if (!stripProcess.waitForFinished())
-            stripProcess.terminate();
+            stripProcess.kill();
     }
 }
 
