@@ -28,12 +28,11 @@
 ****************************************************************************/
 
 #include "attachgdbadapter.h"
-#include "gdbmi.h"
-#include "debuggerstartparameters.h"
 
-#include "gdbengine.h"
-#include "procinterrupt.h"
+#include "debuggerprotocol.h"
 #include "debuggerstringutils.h"
+#include "debuggerstartparameters.h"
+#include "procinterrupt.h"
 
 #include <utils/qtcassert.h>
 
@@ -60,6 +59,12 @@ void GdbAttachEngine::setupEngine()
 {
     QTC_ASSERT(state() == EngineSetupRequested, qDebug() << state());
     showMessage(_("TRYING TO START ADAPTER"));
+
+    if (!startParameters().workingDirectory.isEmpty())
+        m_gdbProc.setWorkingDirectory(startParameters().workingDirectory);
+    if (startParameters().environment.size())
+        m_gdbProc.setEnvironment(startParameters().environment.toStringList());
+
     startGdb();
 }
 
@@ -88,6 +93,7 @@ void GdbAttachEngine::handleAttach(const GdbResponse &response)
     case GdbResultRunning:
         showMessage(_("INFERIOR ATTACHED"));
         showMessage(msgAttachedToStoppedInferior(), StatusBar);
+        tryLoadPythonDumpers();
         handleInferiorPrepared();
         break;
     case GdbResultError:
