@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -31,6 +31,7 @@
 
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/ieditor.h>
+#include <utils/fileutils.h>
 
 #include <QFileInfo>
 #include <QDir>
@@ -38,16 +39,20 @@
 using namespace Core;
 using namespace Locator;
 using namespace Locator::Internal;
+using namespace Utils;
 
 OpenDocumentsFilter::OpenDocumentsFilter(EditorManager *editorManager) :
     m_editorManager(editorManager)
 {
+    setId("Open documents");
+    setDisplayName(tr("Open Documents"));
+    setShortcutString(QString(QLatin1Char('o')));
+    setIncludedByDefault(true);
+
     connect(m_editorManager, SIGNAL(editorOpened(Core::IEditor*)),
             this, SLOT(refreshInternally()));
     connect(m_editorManager, SIGNAL(editorsClosed(QList<Core::IEditor*>)),
             this, SLOT(refreshInternally()));
-    setShortcutString(QString(QLatin1Char('o')));
-    setIncludedByDefault(true);
 }
 
 QList<FilterEntry> OpenDocumentsFilter::matchesFor(QFutureInterface<Locator::FilterEntry> &future, const QString &entry_)
@@ -71,8 +76,8 @@ QList<FilterEntry> OpenDocumentsFilter::matchesFor(QFutureInterface<Locator::Fil
             if (!fileName.isEmpty()) {
                 QFileInfo fi(fileName);
                 FilterEntry fiEntry(this, fi.fileName(), QString(fileName + lineNoSuffix));
-                fiEntry.extraInfo = QDir::toNativeSeparators(fi.path());
-                fiEntry.resolveFileIcon = true;
+                fiEntry.extraInfo = FileUtils::shortNativePath(FileName(fi));
+                fiEntry.fileName = fileName;
                 value.append(fiEntry);
             }
         }
