@@ -49,6 +49,7 @@ class FormEditorScene;
 class NodeInstanceView;
 
 class AbstractFormEditorTool;
+class AbstractCustomTool;
 class MoveTool;
 class SelectionTool;
 class ResizeTool;
@@ -56,7 +57,7 @@ class DragTool;
 class ItemLibraryEntry;
 class QmlItemNode;
 
-class  FormEditorView : public QmlModelView
+class QMLDESIGNERCORE_EXPORT FormEditorView : public QmlModelView
 {
     Q_OBJECT
 
@@ -65,27 +66,29 @@ public:
     ~FormEditorView();
 
     // AbstractView
-    void modelAttached(Model *model);
-    void modelAboutToBeDetached(Model *model);
+    void modelAttached(Model *model) QTC_OVERRIDE;
+    void modelAboutToBeDetached(Model *model) QTC_OVERRIDE;
 
-    void importsChanged(const QList<Import> &addedImports, const QList<Import> &removedImports);
+    void importsChanged(const QList<Import> &addedImports, const QList<Import> &removedImports) QTC_OVERRIDE;
 
-    void nodeCreated(const ModelNode &createdNode);
-    void nodeAboutToBeRemoved(const ModelNode &removedNode);
-    void nodeReparented(const ModelNode &node, const NodeAbstractProperty &newPropertyParent, const NodeAbstractProperty &oldPropertyParent, AbstractView::PropertyChangeFlags propertyChange);
-    void nodeIdChanged(const ModelNode& node, const QString& newId, const QString& oldId);
-    void propertiesAboutToBeRemoved(const QList<AbstractProperty>& propertyList);
-    void variantPropertiesChanged(const QList<VariantProperty>& propertyList, PropertyChangeFlags propertyChange);
-    void bindingPropertiesChanged(const QList<BindingProperty>& propertyList, PropertyChangeFlags propertyChange);
-    void rootNodeTypeChanged(const QString &type, int majorVersion, int minorVersion);
+    void nodeCreated(const ModelNode &createdNode) QTC_OVERRIDE;
+    void nodeAboutToBeRemoved(const ModelNode &removedNode) QTC_OVERRIDE;
+    void nodeReparented(const ModelNode &node, const NodeAbstractProperty &newPropertyParent, const NodeAbstractProperty &oldPropertyParent, AbstractView::PropertyChangeFlags propertyChange) QTC_OVERRIDE;
+    void nodeIdChanged(const ModelNode& node, const QString& newId, const QString& oldId) QTC_OVERRIDE;
+    void propertiesAboutToBeRemoved(const QList<AbstractProperty>& propertyList) QTC_OVERRIDE;
+    void variantPropertiesChanged(const QList<VariantProperty>& propertyList, PropertyChangeFlags propertyChange) QTC_OVERRIDE;
+    void bindingPropertiesChanged(const QList<BindingProperty>& propertyList, PropertyChangeFlags propertyChange) QTC_OVERRIDE;
+    void signalHandlerPropertiesChanged(const QVector<SignalHandlerProperty>& propertyList, PropertyChangeFlags propertyChange) QTC_OVERRIDE;
+    void rootNodeTypeChanged(const QString &type, int majorVersion, int minorVersion) QTC_OVERRIDE;
 
     void selectedNodesChanged(const QList<ModelNode> &selectedNodeList,
-                              const QList<ModelNode> &lastSelectedNodeList);
-    void scriptFunctionsChanged(const ModelNode &node, const QStringList &scriptFunctionList);
-    void propertiesRemoved(const QList<AbstractProperty> &propertyList);
+                              const QList<ModelNode> &lastSelectedNodeList) QTC_OVERRIDE;
+    void scriptFunctionsChanged(const ModelNode &node, const QStringList &scriptFunctionList) QTC_OVERRIDE;
+    void propertiesRemoved(const QList<AbstractProperty> &propertyList) QTC_OVERRIDE;
 
     // FormEditorView
-    QWidget *widget();
+    WidgetInfo widgetInfo() QTC_OVERRIDE;
+
     FormEditorWidget *formEditorWidget();
     AbstractFormEditorTool *currentTool() const;
     FormEditorScene *scene() const;
@@ -97,26 +100,30 @@ public:
     void changeToSelectionTool(QGraphicsSceneMouseEvent *event);
     void changeToResizeTool();
     void changeToTransformTools();
+    void changeToCustomTool(const ModelNode &modelNode);
+    void changeToCustomTool(AbstractCustomTool *customTool);
+
+    void registerTool(AbstractCustomTool *tool);
 
     void nodeSlidedToIndex(const NodeListProperty &listProperty, int newIndex, int oldIndex);
-    void auxiliaryDataChanged(const ModelNode &node, const QString &name, const QVariant &data);
+    void auxiliaryDataChanged(const ModelNode &node, const PropertyName &name, const QVariant &data) QTC_OVERRIDE;
 
-    void instancesCompleted(const QVector<ModelNode> &completedNodeList);
-    void instanceInformationsChange(const QMultiHash<ModelNode, InformationName> &informationChangeHash);
-    void instancesRenderImageChanged(const QVector<ModelNode> &nodeList);
-    void instancesPreviewImageChanged(const QVector<ModelNode> &nodeList);
-    void instancesChildrenChanged(const QVector<ModelNode> &nodeList);
-    void instancePropertyChange(const QList<QPair<ModelNode, QString> > &propertyList);
-    void instancesToken(const QString &tokenName, int tokenNumber, const QVector<ModelNode> &nodeVector);
+    void instancesCompleted(const QVector<ModelNode> &completedNodeList) QTC_OVERRIDE;
+    void instanceInformationsChange(const QMultiHash<ModelNode, InformationName> &informationChangeHash) QTC_OVERRIDE;
+    void instancesRenderImageChanged(const QVector<ModelNode> &nodeList) QTC_OVERRIDE;
+    void instancesPreviewImageChanged(const QVector<ModelNode> &nodeList) QTC_OVERRIDE;
+    void instancesChildrenChanged(const QVector<ModelNode> &nodeList) QTC_OVERRIDE;
+    void instancePropertyChange(const QList<QPair<ModelNode, PropertyName> > &propertyList) QTC_OVERRIDE;
+    void instancesToken(const QString &tokenName, int tokenNumber, const QVector<ModelNode> &nodeVector) QTC_OVERRIDE;
 
-    void rewriterBeginTransaction();
-    void rewriterEndTransaction();
+    void rewriterBeginTransaction() QTC_OVERRIDE;
+    void rewriterEndTransaction() QTC_OVERRIDE;
 
     double margins() const;
     double spacing() const;
     void deActivateItemCreator();
 
-    void actualStateChanged(const ModelNode &node);
+    void actualStateChanged(const ModelNode &node) QTC_OVERRIDE;
 
 protected:
     void reset();
@@ -136,6 +143,7 @@ private: //functions
 private: //variables
     QWeakPointer<FormEditorWidget> m_formEditorWidget;
     QWeakPointer<FormEditorScene> m_scene;
+    QList<AbstractCustomTool*> m_customToolList;
     MoveTool *m_moveTool;
     SelectionTool *m_selectionTool;
     ResizeTool *m_resizeTool;
