@@ -27,19 +27,22 @@
 **
 ****************************************************************************/
 
+#include "texttomodelmerger.h"
 #include "abstractproperty.h"
 #include "bindingproperty.h"
 #include "filemanager/firstdefinitionfinder.h"
 #include "filemanager/objectlengthcalculator.h"
 #include "filemanager/qmlrefactoring.h"
+#include "filemanager/qmlwarningdialog.h"
 #include "rewriteaction.h"
 #include "nodeproperty.h"
 #include "propertyparser.h"
 #include "textmodifier.h"
-#include "texttomodelmerger.h"
 #include "rewriterview.h"
 #include "variantproperty.h"
+#include "signalhandlerproperty.h"
 #include "nodemetainfo.h"
+#include "qmldesignercorelib_global.h"
 
 #include <languageutils/componentversion.h>
 #include <qmljs/qmljsevaluate.h>
@@ -66,7 +69,7 @@ namespace {
 static inline QStringList supportedVersionsList()
 {
     QStringList list;
-    list << QLatin1String("1.0") << QLatin1String("1.1");
+    list << QLatin1String("1.0") << QLatin1String("1.1") << QLatin1String("2.0") << QLatin1String("2.1");
     return list;
 }
 
@@ -138,27 +141,6 @@ static inline QString fixEscapedUnicodeChar(const QString &value) //convert "\u2
     }
     return value;
 }
- 
-static inline int fixUpMajorVersionForQt(const QString &value, int i)
-{
-    if (i == 4 && value == "Qt")
-        return 1;
-    else return i;
-}
-
-static inline int fixUpMinorVersionForQt(const QString &value, int i)
-{
-    if (i == 7 && value == "Qt")
-        return 0;
-    else return i;
-}
-
-static inline QString fixUpPackeNameForQt(const QString &value)
-{
-    if (value == "Qt")
-        return "QtQuick";
-    return value;
-}
 
 static inline bool isSignalPropertyName(const QString &signalName)
 {
@@ -263,9 +245,9 @@ static inline QVariant convertDynamicPropertyValueToVariant(const QString &astVa
     }
 }
 
-static bool isComponentType(const QString &type)
+static bool isComponentType(const QmlDesigner::TypeName &type)
 {
-    return  type == QLatin1String("Component") || type == QLatin1String("Qt.Component") || type == QLatin1String("QtQuick.Component");
+    return  type == "Component" || type == "Qt.Component" || type == "QtQuick.Component";
 }
 
 static bool isCustomParserType(const QString &type)
@@ -277,16 +259,20 @@ static bool isCustomParserType(const QString &type)
 }
 
 
-static bool isPropertyChangesType(const QString &type)
-{    
-    return  type == QLatin1String("PropertyChanges") || type == QLatin1String("QtQuick.PropertyChanges") || type == QLatin1String("Qt.PropertyChanges");
+static bool isPropertyChangesType(const QmlDesigner::TypeName &type)
+{
+    return  type == "PropertyChanges" || type == "QtQuick.PropertyChanges" || type == "Qt.PropertyChanges";
 }
 
-static bool propertyIsComponentType(const QmlDesigner::NodeAbstractProperty &property, const QString &type, QmlDesigner::Model *model)
+static bool isConnectionsType(const QmlDesigner::TypeName &type)
 {
-    if (model->metaInfo(type, -1, -1).isSubclassOf(QLatin1String("QtQuick.Component"), -1, -1) && !isComponentType(type)) {
+    return  type == "Connections" || type == "QtQuick.Connections" || type == "Qt.Connections";
+}
+
+static bool propertyIsComponentType(const QmlDesigner::NodeAbstractProperty &property, const QmlDesigner::TypeName &type, QmlDesigner::Model *model)
+{
+    if (model->metaInfo(type, -1, -1).isSubclassOf("QtQuick.Component", -1, -1) && !isComponentType(type))
         return false; //If the type is already a subclass of Component keep it
-    }
 
     return property.parentModelNode().isValid() &&
             isComponentType(property.parentModelNode().metaInfo().propertyTypeName(property.name()));
@@ -301,16 +287,14 @@ static inline QString extractComponentFromQml(const QString &source)
     if (source.contains("Component")) { //explicit component
         QmlDesigner::FirstDefinitionFinder firstDefinitionFinder(source);
         int offset = firstDefinitionFinder(0);
-        if (offset < 0) {
+        if (offset < 0)
             return QString(); //No object definition found
-        }
         QmlDesigner::ObjectLengthCalculator objectLengthCalculator;
         unsigned length;
-        if (objectLengthCalculator(source, offset, length)) {
+        if (objectLengthCalculator(source, offset, length))
             result = source.mid(offset, length);
-        } else {
+        else
             result = source;
-        }
     } else {
         result = source; //implicit component
     }
@@ -357,11 +341,10 @@ public:
 
         const CppComponentValue * qmlValue = value_cast<CppComponentValue>(value);
         if (qmlValue) {
-            typeName = fixUpPackeNameForQt(qmlValue->moduleName()) + QLatin1String(".") + qmlValue->className();
+            typeName = qmlValue->moduleName() + QLatin1String(".") + qmlValue->className();
 
-            //### todo this is just a hack to support QtQuick 1.0
-            majorVersion = fixUpMajorVersionForQt(qmlValue->moduleName(), qmlValue->componentVersion().majorVersion());
-            minorVersion = fixUpMinorVersionForQt(qmlValue->moduleName(), qmlValue->componentVersion().minorVersion());
+            majorVersion = qmlValue->componentVersion().majorVersion();
+            minorVersion = qmlValue->componentVersion().minorVersion();
         } else {
             for (UiQualifiedId *iter = astTypeNode; iter; iter = iter->next)
                 if (!iter->next && !iter->name.isEmpty())
@@ -550,20 +533,25 @@ public:
             }
         }
 
-        QVariant v(cleanedValue);
+        if (property->asColorValue())
+            return PropertyParser::read(QVariant::Color, cleanedValue);
+        else if (property->asUrlValue())
+            return PropertyParser::read(QVariant::Url, cleanedValue);
+
+        QVariant value(cleanedValue);
         if (property->asBooleanValue()) {
-            v.convert(QVariant::Bool);
-        } else if (property->asColorValue()) {
-            v.convert(QVariant::Color);
+            value.convert(QVariant::Bool);
+            return value;
         } else if (property->asNumberValue()) {
-            v.convert(QVariant::Double);
+            value.convert(QVariant::Double);
+            return value;
         } else if (property->asStringValue()) {
             // nothing to do
         } else { //property alias et al
             if (!hasQuotes)
                 return cleverConvert(cleanedValue);
         }
-        return v;
+        return value;
     }
 
     QVariant convertToEnum(Statement *rhs, const QString &propertyPrefix, UiQualifiedId *propertyId)
@@ -574,9 +562,8 @@ public:
 
         const ObjectValue *containingObject = 0;
         QString name;
-        if (!lookupProperty(propertyPrefix, propertyId, 0, &containingObject, &name)) {
+        if (!lookupProperty(propertyPrefix, propertyId, 0, &containingObject, &name))
             return QVariant();
-        }
 
         if (containingObject)
             containingObject->lookupMember(name, m_context, &containingObject);
@@ -648,16 +635,14 @@ static inline bool smartVeryFuzzyCompare(QVariant value1, QVariant value2)
         if (!ok1 || !ok2)
             return false;
 
-        if (qFuzzyCompare(a, b)) {
+        if (qFuzzyCompare(a, b))
             return true;
-        }
 
         int ai = qRound(a * 1000);
         int bi = qRound(b * 1000);
 
-        if (qFuzzyCompare((qreal(ai) / 1000), (qreal(bi) / 1000))) {
+        if (qFuzzyCompare((qreal(ai) / 1000), (qreal(bi) / 1000)))
             return true;
-        }
     }
     return false;
 }
@@ -762,6 +747,7 @@ bool TextToModelMerger::load(const QString &data, DifferenceHandler &differenceH
         m_document = doc;
 
         QList<RewriterView::Error> errors;
+        QList<RewriterView::Error> warnings;
 
         foreach (const QmlJS::DiagnosticMessage &diagnosticMessage, ctxt.diagnosticLinkMessages()) {
             errors.append(RewriterView::Error(diagnosticMessage, QUrl::fromLocalFile(doc->fileName())));
@@ -770,14 +756,14 @@ bool TextToModelMerger::load(const QString &data, DifferenceHandler &differenceH
         setupImports(doc, differenceHandler);
 
         if (m_rewriterView->model()->imports().isEmpty()) {
-            const QmlJS::DiagnosticMessage diagnosticMessage(QmlJS::DiagnosticMessage::Error, AST::SourceLocation(0, 0, 0, 0), QCoreApplication::translate("QmlDesigner::TextToModelMerger error message", "No import statements found"));
+            const QmlJS::DiagnosticMessage diagnosticMessage(QmlJS::DiagnosticMessage::Error, AST::SourceLocation(0, 0, 0, 0), QCoreApplication::translate("QmlDesigner::TextToModelMerger", "No import statements found"));
             errors.append(RewriterView::Error(diagnosticMessage, QUrl::fromLocalFile(doc->fileName())));
         }
 
         foreach (const QmlDesigner::Import &import, m_rewriterView->model()->imports()) {
             if (import.isLibraryImport() && import.url() == QLatin1String("QtQuick") && !supportedQtQuickVersion(import.version())) {
                 const QmlJS::DiagnosticMessage diagnosticMessage(QmlJS::DiagnosticMessage::Error, AST::SourceLocation(0, 0, 0, 0),
-                                                                 QCoreApplication::translate("QmlDesigner::TextToModelMerger error message", "Unsupported QtQuick version"));
+                                                                 QCoreApplication::translate("QmlDesigner::TextToModelMerger", "Unsupported QtQuick version"));
                 errors.append(RewriterView::Error(diagnosticMessage, QUrl::fromLocalFile(doc->fileName())));
             }
         }
@@ -788,9 +774,26 @@ bool TextToModelMerger::load(const QString &data, DifferenceHandler &differenceH
             check.disableMessage(StaticAnalysis::ErrPrototypeCycle);
             check.disableMessage(StaticAnalysis::ErrCouldNotResolvePrototype);
             check.disableMessage(StaticAnalysis::ErrCouldNotResolvePrototypeOf);
+
+            foreach (StaticAnalysis::Type type, StaticAnalysis::Message::allMessageTypes()) {
+                StaticAnalysis::PrototypeMessageData prototypeMessageData = StaticAnalysis::Message::prototypeForMessageType(type);
+                if (prototypeMessageData.severity == StaticAnalysis::MaybeWarning
+                        || prototypeMessageData.severity == StaticAnalysis::Warning) {
+                    check.disableMessage(type);
+                }
+            }
+
+            check.enableMessage(StaticAnalysis::WarnImperativeCodeNotEditableInVisualDesigner);
+            check.enableMessage(StaticAnalysis::WarnUnsupportedTypeInVisualDesigner);
+            check.enableMessage(StaticAnalysis::WarnReferenceToParentItemNotSupportedByVisualDesigner);
+            check.enableMessage(StaticAnalysis::WarnUndefinedValueForVisualDesigner);
+            check.enableMessage(StaticAnalysis::WarnStatesOnlyInRootItemForVisualDesigner);
+
             foreach (const StaticAnalysis::Message &message, check()) {
                 if (message.severity == StaticAnalysis::Error)
                     errors.append(RewriterView::Error(message.toDiagnosticMessage(), QUrl::fromLocalFile(doc->fileName())));
+                if (message.severity == StaticAnalysis::Warning)
+                    warnings.append(RewriterView::Error(message.toDiagnosticMessage(), QUrl::fromLocalFile(doc->fileName())));
             }
 
             if (!errors.isEmpty()) {
@@ -798,6 +801,27 @@ bool TextToModelMerger::load(const QString &data, DifferenceHandler &differenceH
                 setActive(false);
                 return false;
             }
+
+            if (!warnings.isEmpty() && differenceHandler.isValidator()) {
+
+                QString title = QCoreApplication::translate("QmlDesigner::TextToModelMerger", "This .qml file contains features "
+                                                            "which are not supported by Qt Quick Designer");
+
+                QStringList message;
+
+                foreach (const RewriterView::Error &warning, warnings) {
+                    QString string = QLatin1String("Line: ") +  QString::number(warning.line()) + QLatin1String(": ")  + warning.description();
+                    //string += QLatin1String(" <a href=\"") + QString::number(warning.line()) + QLatin1String("\">Go to error</a>") + QLatin1String("<p>");
+                    message << string;
+                }
+
+                QmlWarningDialog warningDialog(0, message);
+                if (warningDialog.warningsEnabled() && warningDialog.exec()) {
+                    m_rewriterView->setErrors(warnings);
+                    setActive(false);
+                    return false;
+                }
+            }
         }
 
         UiObjectMember *astRootNode = 0;
@@ -838,10 +862,15 @@ void TextToModelMerger::syncNode(ModelNode &modelNode,
 
     m_rewriterView->positionStorage()->setNodeOffset(modelNode, astObjectType->identifierToken.offset);
 
-    QString typeName, defaultPropertyName;
+    QString typeNameString;
+    QString defaultPropertyNameString;
     int majorVersion;
     int minorVersion;
-    context->lookup(astObjectType, typeName, majorVersion, minorVersion, defaultPropertyName);
+    context->lookup(astObjectType, typeNameString, majorVersion, minorVersion, defaultPropertyNameString);
+
+    TypeName typeName = typeNameString.toUtf8();
+    PropertyName defaultPropertyName = defaultPropertyNameString.toUtf8();
+
     if (defaultPropertyName.isEmpty()) //fallback and use the meta system of the model
         defaultPropertyName = modelNode.metaInfo().defaultPropertyName();
 
@@ -874,16 +903,16 @@ void TextToModelMerger::syncNode(ModelNode &modelNode,
     }
 
     if (isComponentType(typeName) || isImplicitComponent)
-        setupComponentDelayed(modelNode, !differenceHandler.isValidator());
+        setupComponentDelayed(modelNode, differenceHandler.isValidator());
 
     if (isCustomParserType(typeName))
-        setupCustomParserNodeDelayed(modelNode, !differenceHandler.isValidator());
+        setupCustomParserNodeDelayed(modelNode, differenceHandler.isValidator());
 
     context->enterScope(astNode);
 
-    QSet<QString> modelPropertyNames = QSet<QString>::fromList(modelNode.propertyNames());
+    QSet<PropertyName> modelPropertyNames = QSet<PropertyName>::fromList(modelNode.propertyNames());
     if (!modelNode.id().isEmpty())
-        modelPropertyNames.insert(QLatin1String("id"));
+        modelPropertyNames.insert("id");
     QList<UiObjectMember *> defaultPropertyItems;
 
     for (UiObjectMemberList *iter = astInitializer->members; iter; iter = iter->next) {
@@ -893,15 +922,15 @@ void TextToModelMerger::syncNode(ModelNode &modelNode,
 
         if (UiArrayBinding *array = cast<UiArrayBinding *>(member)) {
             const QString astPropertyName = toString(array->qualifiedId);
-            if (isPropertyChangesType(typeName) || context->lookupProperty(QString(), array->qualifiedId)) {
-                AbstractProperty modelProperty = modelNode.property(astPropertyName);
+            if (isPropertyChangesType(typeName) || isConnectionsType(typeName) || context->lookupProperty(QString(), array->qualifiedId)) {
+                AbstractProperty modelProperty = modelNode.property(astPropertyName.toUtf8());
                 QList<UiObjectMember *> arrayMembers;
                 for (UiArrayMemberList *iter = array->members; iter; iter = iter->next)
                     if (UiObjectMember *member = iter->member)
                         arrayMembers.append(member);
 
                 syncArrayProperty(modelProperty, arrayMembers, context, differenceHandler);
-                modelPropertyNames.remove(astPropertyName);
+                modelPropertyNames.remove(astPropertyName.toUtf8());
             } else {
                 qWarning() << "Skipping invalid array property" << astPropertyName
                            << "for node type" << modelNode.type();
@@ -915,7 +944,7 @@ void TextToModelMerger::syncNode(ModelNode &modelNode,
                                                           context,
                                                           differenceHandler);
                 foreach (const QString &prop, props)
-                    modelPropertyNames.remove(prop);
+                    modelPropertyNames.remove(prop.toUtf8());
             } else {
                 defaultPropertyItems.append(member);
             }
@@ -927,14 +956,15 @@ void TextToModelMerger::syncNode(ModelNode &modelNode,
                 const Value *propertyType = 0;
                 const ObjectValue *containingObject = 0;
                 QString name;
-                if (context->lookupProperty(QString(), binding->qualifiedId, &propertyType, &containingObject, &name) || isPropertyChangesType(typeName)) {
-                    AbstractProperty modelProperty = modelNode.property(astPropertyName);
-                    if (context->isArrayProperty(propertyType, containingObject, name)) {
+                if (context->lookupProperty(QString(), binding->qualifiedId, &propertyType, &containingObject, &name)
+                        || isPropertyChangesType(typeName)
+                        || isConnectionsType(typeName)) {
+                    AbstractProperty modelProperty = modelNode.property(astPropertyName.toUtf8());
+                    if (context->isArrayProperty(propertyType, containingObject, name))
                         syncArrayProperty(modelProperty, QList<QmlJS::AST::UiObjectMember*>() << member, context, differenceHandler);
-                    } else {
+                    else
                         syncNodeProperty(modelProperty, binding, context, differenceHandler);
-                    }
-                    modelPropertyNames.remove(astPropertyName);
+                    modelPropertyNames.remove(astPropertyName.toUtf8());
                 } else {
                     qWarning() << "Skipping invalid node property" << astPropertyName
                                << "for node type" << modelNode.type();
@@ -955,15 +985,15 @@ void TextToModelMerger::syncNode(ModelNode &modelNode,
                 astValue = textAt(context->doc(),
                                   property->statement->firstSourceLocation(),
                                   property->statement->lastSourceLocation());
-            const QString &astType = property->memberType.toString();
-            AbstractProperty modelProperty = modelNode.property(astName);
+            const TypeName &astType = property->memberType.toUtf8();
+            AbstractProperty modelProperty = modelNode.property(astName.toUtf8());
             if (!property->statement || isLiteralValue(property->statement)) {
                 const QVariant variantValue = convertDynamicPropertyValueToVariant(astValue, astType);
                 syncVariantProperty(modelProperty, variantValue, astType, differenceHandler);
             } else {
                 syncExpressionProperty(modelProperty, astValue, astType, differenceHandler);
             }
-            modelPropertyNames.remove(astName);
+            modelPropertyNames.remove(astName.toUtf8());
         } else {
             qWarning() << "Found an unknown QML value.";
         }
@@ -971,7 +1001,7 @@ void TextToModelMerger::syncNode(ModelNode &modelNode,
 
     if (!defaultPropertyItems.isEmpty()) {
         if (isComponentType(modelNode.type()))
-            setupComponentDelayed(modelNode, !differenceHandler.isValidator());
+            setupComponentDelayed(modelNode, differenceHandler.isValidator());
         if (defaultPropertyName.isEmpty()) {
             qWarning() << "No default property for node type" << modelNode.type() << ", ignoring child items.";
         } else {
@@ -989,11 +1019,11 @@ void TextToModelMerger::syncNode(ModelNode &modelNode,
         }
     }
 
-    foreach (const QString &modelPropertyName, modelPropertyNames) {
+    foreach (const PropertyName &modelPropertyName, modelPropertyNames) {
         AbstractProperty modelProperty = modelNode.property(modelPropertyName);
 
         // property deleted.
-        if (modelPropertyName == QLatin1String("id"))
+        if (modelPropertyName == "id")
             differenceHandler.idsDiffer(modelNode, QString());
         else
             differenceHandler.propertyAbsentFromQml(modelProperty);
@@ -1002,7 +1032,7 @@ void TextToModelMerger::syncNode(ModelNode &modelNode,
     context->leaveScope();
 }
 
-QString TextToModelMerger::syncScriptBinding(ModelNode &modelNode,
+QmlDesigner::PropertyName TextToModelMerger::syncScriptBinding(ModelNode &modelNode,
                                              const QString &prefix,
                                              UiScriptBinding *script,
                                              ReadingContext *context,
@@ -1025,46 +1055,52 @@ QString TextToModelMerger::syncScriptBinding(ModelNode &modelNode,
 
     if (astPropertyName == QLatin1String("id")) {
         syncNodeId(modelNode, astValue, differenceHandler);
-        return astPropertyName;
+        return astPropertyName.toUtf8();
     }
 
-    if (isSignalPropertyName(astPropertyName))
-        return QString();
+    if (isSignalPropertyName(astPropertyName)) {
+        AbstractProperty modelProperty = modelNode.property(astPropertyName.toUtf8());
+        syncSignalHandler(modelProperty, astValue, differenceHandler);
+        return astPropertyName.toUtf8();
+    }
 
     if (isLiteralValue(script)) {
-        if (isPropertyChangesType(modelNode.type())) {
-            AbstractProperty modelProperty = modelNode.property(astPropertyName);
+        if (isPropertyChangesType(modelNode.type())
+                || isConnectionsType(modelNode.type())) {
+            AbstractProperty modelProperty = modelNode.property(astPropertyName.toUtf8());
             const QVariant variantValue(deEscape(stripQuotes(astValue)));
-            syncVariantProperty(modelProperty, variantValue, QString(), differenceHandler);
-            return astPropertyName;
+            syncVariantProperty(modelProperty, variantValue, TypeName(), differenceHandler);
+            return astPropertyName.toUtf8();
         } else {
             const QVariant variantValue = context->convertToVariant(astValue, prefix, script->qualifiedId);
             if (variantValue.isValid()) {
-                AbstractProperty modelProperty = modelNode.property(astPropertyName);
-                syncVariantProperty(modelProperty, variantValue, QString(), differenceHandler);
-                return astPropertyName;
+                AbstractProperty modelProperty = modelNode.property(astPropertyName.toUtf8());
+                syncVariantProperty(modelProperty, variantValue, TypeName(), differenceHandler);
+                return astPropertyName.toUtf8();
             } else {
                 qWarning() << "Skipping invalid variant property" << astPropertyName
                            << "for node type" << modelNode.type();
-                return QString();
+                return PropertyName();
             }
         }
     }
 
     const QVariant enumValue = context->convertToEnum(script->statement, prefix, script->qualifiedId);
     if (enumValue.isValid()) { // It is a qualified enum:
-        AbstractProperty modelProperty = modelNode.property(astPropertyName);
-        syncVariantProperty(modelProperty, enumValue, QString(), differenceHandler); // TODO: parse type
-        return astPropertyName;
+        AbstractProperty modelProperty = modelNode.property(astPropertyName.toUtf8());
+        syncVariantProperty(modelProperty, enumValue, TypeName(), differenceHandler); // TODO: parse type
+        return astPropertyName.toUtf8();
     } else { // Not an enum, so:
-        if (isPropertyChangesType(modelNode.type()) || context->lookupProperty(prefix, script->qualifiedId)) {
-            AbstractProperty modelProperty = modelNode.property(astPropertyName);
-            syncExpressionProperty(modelProperty, astValue, QString(), differenceHandler); // TODO: parse type
-            return astPropertyName;
+        if (isPropertyChangesType(modelNode.type())
+                || isConnectionsType(modelNode.type())
+                || context->lookupProperty(prefix, script->qualifiedId)) {
+            AbstractProperty modelProperty = modelNode.property(astPropertyName.toUtf8());
+            syncExpressionProperty(modelProperty, astValue, TypeName(), differenceHandler); // TODO: parse type
+            return astPropertyName.toUtf8();
         } else {
             qWarning() << "Skipping invalid expression property" << astPropertyName
                     << "for node type" << modelNode.type();
-            return QString();
+            return PropertyName();
         }
     }
 }
@@ -1094,10 +1130,13 @@ void TextToModelMerger::syncNodeProperty(AbstractProperty &modelProperty,
                                          ReadingContext *context,
                                          DifferenceHandler &differenceHandler)
 {
-    QString typeName, dummy;
+    QString typeNameString;
+    QString dummy;
     int majorVersion;
     int minorVersion;
-    context->lookup(binding->qualifiedTypeNameId, typeName, majorVersion, minorVersion, dummy);
+    context->lookup(binding->qualifiedTypeNameId, typeNameString, majorVersion, minorVersion, dummy);
+
+    TypeName typeName = typeNameString.toUtf8();
 
     if (typeName.isEmpty()) {
         qWarning() << "Skipping node with unknown type" << toString(binding->qualifiedTypeNameId);
@@ -1112,13 +1151,14 @@ void TextToModelMerger::syncNodeProperty(AbstractProperty &modelProperty,
                                                typeName,
                                                majorVersion,
                                                minorVersion,
-                                               binding, context);
+                                               binding,
+                                               context);
     }
 }
 
 void TextToModelMerger::syncExpressionProperty(AbstractProperty &modelProperty,
                                                const QString &javascript,
-                                               const QString &astType,
+                                               const TypeName &astType,
                                                DifferenceHandler &differenceHandler)
 {
     if (modelProperty.isBindingProperty()) {
@@ -1133,6 +1173,21 @@ void TextToModelMerger::syncExpressionProperty(AbstractProperty &modelProperty,
     }
 }
 
+void TextToModelMerger::syncSignalHandler(AbstractProperty &modelProperty,
+                                               const QString &javascript,
+                                               DifferenceHandler &differenceHandler)
+{
+    if (modelProperty.isSignalHandlerProperty()) {
+        SignalHandlerProperty signalHandlerProperty = modelProperty.toSignalHandlerProperty();
+        if (signalHandlerProperty.source() != javascript) {
+            differenceHandler.signalHandlerSourceDiffer(signalHandlerProperty, javascript);
+        }
+    } else {
+        differenceHandler.shouldBeSignalHandlerProperty(modelProperty, javascript);
+    }
+}
+
+
 void TextToModelMerger::syncArrayProperty(AbstractProperty &modelProperty,
                                           const QList<UiObjectMember *> &arrayMembers,
                                           ReadingContext *context,
@@ -1150,7 +1205,7 @@ void TextToModelMerger::syncArrayProperty(AbstractProperty &modelProperty,
 
 void TextToModelMerger::syncVariantProperty(AbstractProperty &modelProperty,
                                             const QVariant &astValue,
-                                            const QString &astType,
+                                            const TypeName &astType,
                                             DifferenceHandler &differenceHandler)
 {
     if (modelProperty.isVariantProperty()) {
@@ -1195,7 +1250,7 @@ void TextToModelMerger::syncNodeListProperty(NodeListProperty &modelListProperty
     }
 }
 
-ModelNode TextToModelMerger::createModelNode(const QString &typeName,
+ModelNode TextToModelMerger::createModelNode(const TypeName &typeName,
                                              int majorVersion,
                                              int minorVersion,
                                              bool isImplicitComponent,
@@ -1276,7 +1331,7 @@ void ModelValidator::importAbsentInQMl(const QmlDesigner::Import &import)
 
 void ModelValidator::bindingExpressionsDiffer(BindingProperty &modelProperty,
                                               const QString &javascript,
-                                              const QString &astType)
+                                              const TypeName &astType)
 {
     Q_UNUSED(modelProperty)
     Q_UNUSED(javascript)
@@ -1288,13 +1343,28 @@ void ModelValidator::bindingExpressionsDiffer(BindingProperty &modelProperty,
 
 void ModelValidator::shouldBeBindingProperty(AbstractProperty &modelProperty,
                                              const QString &/*javascript*/,
-                                             const QString &/*astType*/)
+                                             const TypeName &/*astType*/)
 {
     Q_UNUSED(modelProperty)
     Q_ASSERT(modelProperty.isBindingProperty());
     Q_ASSERT(0);
 }
 
+void ModelValidator::signalHandlerSourceDiffer(SignalHandlerProperty &modelProperty, const QString &javascript)
+{
+    Q_UNUSED(modelProperty)
+    Q_UNUSED(javascript)
+    Q_ASSERT(modelProperty.source() == javascript);
+    Q_ASSERT(0);
+}
+
+void ModelValidator::shouldBeSignalHandlerProperty(AbstractProperty &modelProperty, const QString & /*javascript*/)
+{
+    Q_UNUSED(modelProperty)
+    Q_ASSERT(modelProperty.isSignalHandlerProperty());
+    Q_ASSERT(0);
+}
+
 void ModelValidator::shouldBeNodeListProperty(AbstractProperty &modelProperty,
                                               const QList<UiObjectMember *> /*arrayMembers*/,
                                               ReadingContext * /*context*/)
@@ -1304,7 +1374,7 @@ void ModelValidator::shouldBeNodeListProperty(AbstractProperty &modelProperty,
     Q_ASSERT(0);
 }
 
-void ModelValidator::variantValuesDiffer(VariantProperty &modelProperty, const QVariant &qmlVariantValue, const QString &dynamicTypeName)
+void ModelValidator::variantValuesDiffer(VariantProperty &modelProperty, const QVariant &qmlVariantValue, const TypeName &dynamicTypeName)
 {
     Q_UNUSED(modelProperty)
     Q_UNUSED(qmlVariantValue)
@@ -1319,7 +1389,7 @@ void ModelValidator::variantValuesDiffer(VariantProperty &modelProperty, const Q
     Q_ASSERT(0);
 }
 
-void ModelValidator::shouldBeVariantProperty(AbstractProperty &modelProperty, const QVariant &/*qmlVariantValue*/, const QString &/*dynamicTypeName*/)
+void ModelValidator::shouldBeVariantProperty(AbstractProperty &modelProperty, const QVariant &/*qmlVariantValue*/, const TypeName &/*dynamicTypeName*/)
 {
     Q_UNUSED(modelProperty)
 
@@ -1328,7 +1398,7 @@ void ModelValidator::shouldBeVariantProperty(AbstractProperty &modelProperty, co
 }
 
 void ModelValidator::shouldBeNodeProperty(AbstractProperty &modelProperty,
-                                          const QString &/*typeName*/,
+                                          const TypeName &/*typeName*/,
                                           int /*majorVersion*/,
                                           int /*minorVersion*/,
                                           UiObjectMember * /*astNode*/,
@@ -1358,7 +1428,7 @@ ModelNode ModelValidator::listPropertyMissingModelNode(NodeListProperty &/*model
 
 void ModelValidator::typeDiffers(bool /*isRootNode*/,
                                  ModelNode &modelNode,
-                                 const QString &typeName,
+                                 const TypeName &typeName,
                                  int majorVersion,
                                  int minorVersion,
                                  QmlJS::AST::UiObjectMember * /*astNode*/,
@@ -1404,26 +1474,36 @@ void ModelAmender::importAbsentInQMl(const QmlDesigner::Import &import)
 
 void ModelAmender::bindingExpressionsDiffer(BindingProperty &modelProperty,
                                             const QString &javascript,
-                                            const QString &astType)
+                                            const TypeName &astType)
 {
-    if (astType.isEmpty()) {
+    if (astType.isEmpty())
         modelProperty.setExpression(javascript);
-    } else {
+    else
         modelProperty.setDynamicTypeNameAndExpression(astType, javascript);
-    }
 }
 
 void ModelAmender::shouldBeBindingProperty(AbstractProperty &modelProperty,
                                            const QString &javascript,
-                                           const QString &astType)
+                                           const TypeName &astType)
 {
     ModelNode theNode = modelProperty.parentModelNode();
     BindingProperty newModelProperty = theNode.bindingProperty(modelProperty.name());
-    if (astType.isEmpty()) {
+    if (astType.isEmpty())
         newModelProperty.setExpression(javascript);
-    } else {
+    else
         newModelProperty.setDynamicTypeNameAndExpression(astType, javascript);
-    }
+}
+
+void ModelAmender::signalHandlerSourceDiffer(SignalHandlerProperty &modelProperty, const QString &javascript)
+{
+    modelProperty.setSource(javascript);
+}
+
+void ModelAmender::shouldBeSignalHandlerProperty(AbstractProperty &modelProperty, const QString &javascript)
+{
+    ModelNode theNode = modelProperty.parentModelNode();
+    SignalHandlerProperty newModelProperty = theNode.signalHandlerProperty(modelProperty.name());
+    newModelProperty.setSource(javascript);
 }
 
 void ModelAmender::shouldBeNodeListProperty(AbstractProperty &modelProperty,
@@ -1440,7 +1520,7 @@ void ModelAmender::shouldBeNodeListProperty(AbstractProperty &modelProperty,
 
 
 
-void ModelAmender::variantValuesDiffer(VariantProperty &modelProperty, const QVariant &qmlVariantValue, const QString &dynamicType)
+void ModelAmender::variantValuesDiffer(VariantProperty &modelProperty, const QVariant &qmlVariantValue, const TypeName &dynamicType)
 {
 //    qDebug()<< "ModelAmender::variantValuesDiffer for property"<<modelProperty.name()
 //            << "in node" << modelProperty.parentModelNode().id()
@@ -1453,7 +1533,7 @@ void ModelAmender::variantValuesDiffer(VariantProperty &modelProperty, const QVa
         modelProperty.setDynamicTypeNameAndValue(dynamicType, qmlVariantValue);
 }
 
-void ModelAmender::shouldBeVariantProperty(AbstractProperty &modelProperty, const QVariant &qmlVariantValue, const QString &dynamicTypeName)
+void ModelAmender::shouldBeVariantProperty(AbstractProperty &modelProperty, const QVariant &qmlVariantValue, const TypeName &dynamicTypeName)
 {
     ModelNode theNode = modelProperty.parentModelNode();
     VariantProperty newModelProperty = theNode.variantProperty(modelProperty.name());
@@ -1465,7 +1545,7 @@ void ModelAmender::shouldBeVariantProperty(AbstractProperty &modelProperty, cons
 }
 
 void ModelAmender::shouldBeNodeProperty(AbstractProperty &modelProperty,
-                                        const QString &typeName,
+                                        const TypeName &typeName,
                                         int majorVersion,
                                         int minorVersion,
                                         UiObjectMember *astNode,
@@ -1513,10 +1593,14 @@ ModelNode ModelAmender::listPropertyMissingModelNode(NodeListProperty &modelProp
     if (!astObjectType || !astInitializer)
         return ModelNode();
 
-    QString typeName, fullTypeName, dummy;
+    QString typeNameString;
+    QString fullTypeName;
+    QString dummy;
     int majorVersion;
     int minorVersion;
-    context->lookup(astObjectType, typeName, majorVersion, minorVersion, dummy);
+    context->lookup(astObjectType, typeNameString, majorVersion, minorVersion, dummy);
+
+    TypeName typeName = typeNameString.toUtf8();
 
     if (typeName.isEmpty()) {
         qWarning() << "Skipping node with unknown type" << toString(astObjectType);
@@ -1543,7 +1627,7 @@ ModelNode ModelAmender::listPropertyMissingModelNode(NodeListProperty &modelProp
             modelProperty.reparentHere(newNode);
         } else { //The default property could a NodeProperty implicitly (delegate:)
             modelProperty.parentModelNode().removeProperty(modelProperty.name());
-            modelProperty.reparentHere(newNode); 
+            modelProperty.reparentHere(newNode);
         }
     } else {
         modelProperty.reparentHere(newNode);
@@ -1553,7 +1637,7 @@ ModelNode ModelAmender::listPropertyMissingModelNode(NodeListProperty &modelProp
 
 void ModelAmender::typeDiffers(bool isRootNode,
                                ModelNode &modelNode,
-                               const QString &typeName,
+                               const TypeName &typeName,
                                int majorVersion,
                                int minorVersion,
                                QmlJS::AST::UiObjectMember *astNode,
@@ -1611,9 +1695,8 @@ void TextToModelMerger::setupComponent(const ModelNode &node)
 
     QString result = extractComponentFromQml(componentText);
 
-    if (result.isEmpty()) {
+    if (result.isEmpty())
         return; //No object definition found
-    }
 
     if (node.nodeSource() != result)
         ModelNode(node).setNodeSource(result);
