@@ -12,14 +12,17 @@ def main():
     templateDir = prepareTemplate(sourceExample)
     examplePath = os.path.join(templateDir, proFile)
     startApplication("qtcreator" + SettingsPath)
+    if not startedWithoutPluginError():
+        return
     # open example project
     openQmakeProject(examplePath)
     # wait for parsing to complete
-    waitForSignal("{type='CppTools::Internal::CppModelManager' unnamed='1'}", "sourceFilesRefreshed(QStringList)")
-    test.verify(waitForObjectItem(":Qt Creator_Utils::NavigationTreeView", "propertyanimation"),
-                "Verifying if: Project is opened.")
+    progressBarWait(30000)
     # open .cpp file in editor
-    doubleClickItem(":Qt Creator_Utils::NavigationTreeView", "propertyanimation.Sources.main\\.cpp", 5, 5, 0, Qt.LeftButton)
+    if not openDocument("propertyanimation.Sources.main\\.cpp"):
+        test.fatal("Could not open main.cpp")
+        invokeMenuItem("File", "Exit")
+        return
     test.verify(checkIfObjectExists(":Qt Creator_CppEditor::Internal::CPPEditorWidget"),
                 "Verifying if: .cpp file is opened in Edit mode.")
     # select some word for example "viewer" and press Ctrl+F.
